@@ -170,7 +170,7 @@ proc convert_to_pdf {filename dest} {
 proc convert_to_png {filename dest} {
   global to_png tcl_debug
   # puts "---> $to_png $filename $destfile"
-  if { ![catch "exec $to_png $filename $dest" msg] } {
+  if { ![catch "exec $to_png $filename png:$dest" msg] } {
     # conversion succeeded, so remove original .xpm file
     if { ![xschem get debug_var] } {
       file delete $filename
@@ -188,6 +188,28 @@ proc key_binding {  s  d } {
 #                          ButtonPress-4
 #
   regsub {.*-} $d {} key
+
+
+  switch $key {
+     Insert { set keysym 65379 } 
+     Escape { set keysym 65307 } 
+     Return { set keysym 65293 } 
+     Delete { set keysym 65535 } 
+     F1 { set keysym  65470 } 
+     F2 { set keysym  65471 } 
+     F3 { set keysym  65472 } 
+     F4 { set keysym  65473 } 
+     F5 { set keysym  65474 } 
+     F6 { set keysym  65475 } 
+     F7 { set keysym  65476 } 
+     F8 { set keysym  65477 } 
+     F9 { set keysym  65478 } 
+     F10 { set keysym  65479 } 
+     F11 { set keysym  65480 } 
+     F12 { set keysym  65481 } 
+     BackSpace { set keysym 65288 } 
+     default { set keysym [scan "$key" %c] }
+  }
   set state 0
   # not found any portable way to get modifier constants ...
   if { [regexp {(Mod1|Alt)-} $d] } { set state [expr $state +8] }
@@ -203,7 +225,7 @@ proc key_binding {  s  d } {
     if {![string compare $d {} ] } {
       bind .drw "<${s}>" {}
     } else {
-      bind .drw  "<${s}>" "xschem callback %T %x %y [scan "$key" %c] 0 0 $state"
+      bind .drw  "<${s}>" "xschem callback %T %x %y $keysym 0 0 $state"
     }
   }
 
@@ -397,18 +419,22 @@ proc edit_netlist {schname } {
 #   INITIALINSTDIR  for instance placement
 # ext:  .sch or .sym or .sch.sym or .sym.sch
 #
-proc save_file_dialog { msg ext global_initdir {initialfile {}} } {
+proc save_file_dialog { msg ext global_initdir {initialfile {}} {overwrt 1} } {
   global tcl_version
   upvar #0 $global_initdir initdir
-
-  set initialdir [file dirname $initialfile]
-  set initialfile [file tail $initialfile]
-  set types(.sym) { {{Symbol files} {.sym}} }
-  set types(.sch) { {{Schematic files} {.sch}} }
-  set types(.sch.sym) { {{Schematic files} {.sch}} {{Symbol files} {.sym}} }
-  set types(.sym.sch) { {{Symbol files} {.sym}} {{Schematic files} {.sch}} }
+  if { $initialfile ne {}} {
+    set initialdir [file dirname $initialfile]
+    set initialfile [file tail $initialfile]
+  } else {
+    set initialdir $initdir
+    set initialfile {}
+  }
+  set types(.sym) { {{All Files} * } {{Symbol files} {.sym}} }
+  set types(.sch) { {{All Files} * } {{Schematic files} {.sch}} }
+  set types(.sch.sym) { {{All Files} * } {{Schematic files} {.sch}} {{Symbol files} {.sym}} }
+  set types(.sym.sch) { {{All Files} * } {{Symbol files} {.sym}} {{Schematic files} {.sch}} }
   if {$tcl_version > 8.5} {
-    set r [tk_getSaveFile -title $msg -initialfile $initialfile -filetypes $types($ext) -initialdir $initialdir -confirmoverwrite 1]
+    set r [tk_getSaveFile -title $msg -initialfile $initialfile -filetypes $types($ext) -initialdir $initialdir -confirmoverwrite $overwrt]
   } else {
     set r [tk_getSaveFile -title $msg -initialfile $initialfile -filetypes $types($ext) -initialdir $initialdir]
   }
@@ -420,7 +446,7 @@ proc save_file_dialog { msg ext global_initdir {initialfile {}} } {
 
 
 proc list_dirs {pathlist } {
-  global list_dirs_selected_dir
+  global list_dirs_selected_dir INITIALINSTDIR
   toplevel .list -class dialog
   wm title .list {Select Library:}
   wm protocol .list WM_DELETE_WINDOW { set list_dirs_selected_dir {} } 
@@ -444,6 +470,12 @@ proc list_dirs {pathlist } {
     pack .list.${x} -side top -fill x 
     incr x
   }
+  frame .list.${x}
+  label .list.${x}.l -text [expr $x+1] -width 4
+  button .list.${x}.b -text {Last used dir} -command "set list_dirs_selected_dir $INITIALINSTDIR"
+  pack .list.${x}.l -side left
+  pack .list.${x}.b -side left -fill x -expand yes
+  pack .list.${x} -side top -fill x 
   frame .list.but
   button .list.but.cancel -text Cancel -command {set list_dirs_selected_dir {} }
   
@@ -460,15 +492,15 @@ proc list_dirs {pathlist } {
 #   INITIALINSTDIR  for instance placement
 # ext:  .sch or .sym or .sch.sym
 #
-proc load_file_dialog { msg ext global_initdir} {
-  global pathlist
+proc xxload_file_dialog { msg ext global_initdir} {
+  global pathlist use_list_dirs
   upvar #0 $global_initdir initdir
-  set types(.sym) { {{Symbol files} {.sym}} }
-  set types(.sch) { {{Schematic files} {.sch}} }
-  set types(.sch.sym) { {{Schematic files} {.sch}} {{Symbol files} {.sym}} }
-  set types(.sym.sch) { {{Symbol files} {.sym}} {{Schematic files} {.sch}} }
+  set types(.sym) { {{All Files} * } {{Symbol files} {.sym}} }
+  set types(.sch) { {{All Files} * } {{Schematic files} {.sch}} }
+  set types(.sch.sym) { {{All Files} * } {{Schematic files} {.sch}} {{Symbol files} {.sym}} }
+  set types(.sym.sch) { {{All Files} * } {{Symbol files} {.sym}} {{Schematic files} {.sch}} }
 
-  if { $ext eq {.sym} } {
+  if { $use_list_dirs && $ext eq {.sym} } {
     set initdir [list_dirs $pathlist]
   }
   if { $initdir eq {} } { return {} } 
@@ -480,10 +512,181 @@ proc load_file_dialog { msg ext global_initdir} {
   return [file normalize $r]
 }
 
+proc myload_set_colors {} {
+  global myload_index1 myload_files2
+  update
+  set dir1 [abs_sym_path [.myload.l.listbox1.list get $myload_index1]]
+  for {set i 0} { $i< [.myload.l.listbox2.list index end] } { incr i} {
+    if {[ file isdirectory "$dir1/[lindex $myload_files2 $i]"]} {
+      .myload.l.listbox2.list itemconfigure $i -foreground blue
+    } else {
+      .myload.l.listbox2.list itemconfigure $i -foreground black
+    }
+  }
+}
+proc myload_set_home {dir} {
+  global pathlist  myload_files1 myload_index1
+
+  set i [lsearch -exact $pathlist $dir]
+  if { $i>=0 } {
+    set myload_files1 $pathlist
+    set myload_index1 $i
+    .myload.l.listbox1.list selection set $myload_index1
+
+  } else {
+    set myload_files1 $dir
+    set myload_index1 0
+    .myload.l.listbox1.list selection set 0
+  }
+
+}
+
+proc load_file_dialog {{msg {}}  {ext {}} {global_initdir {INITIALINSTDIR}}} {
+  global myload_index1 myload_files2 myload_files1 myload_retval myload_dir1 pathlist
+  global myload_default_geometry
+  upvar #0 $global_initdir initdir
+  toplevel .myload -class dialog
+  wm title .myload $msg
+  
+  set_ne myload_index1 0
+  if { ![info exists myload_files1]} {
+    set myload_files1 $pathlist
+    set myload_index1 0
+  }
+  set_ne myload_files2 {}
+
+  # return value
+  set myload_retval {} 
+
+  # set files [lsort [glob -directory . -tails \{.*,*\}]]
+  
+  panedwindow  .myload.l -orient horizontal
+
+  frame .myload.l.listbox1
+  listbox .myload.l.listbox1.list -width 50 -listvariable myload_files1 \
+    -yscrollcommand ".myload.l.listbox1.yscroll set" -selectmode browse \
+    -xscrollcommand ".myload.l.listbox1.xscroll set"
+  scrollbar .myload.l.listbox1.yscroll -command ".myload.l.listbox1.list yview" 
+  scrollbar .myload.l.listbox1.xscroll -command ".myload.l.listbox1.list xview" -orient horiz
+  pack  .myload.l.listbox1.yscroll -side right -fill y
+  pack  .myload.l.listbox1.xscroll -side bottom -fill x
+  pack  .myload.l.listbox1.list -fill both -expand true
+  bind .myload.l.listbox1.list <<ListboxSelect>> { 
+    set sel [.myload.l.listbox1.list curselection]
+    if { $sel ne {} } {
+      set myload_dir1 [abs_sym_path [.myload.l.listbox1.list get $sel]]
+      set myload_index1 $sel
+      set myload_files2 [lsort [glob -directory $myload_dir1 -tails \{.*,*\}]]
+      myload_set_colors
+    }
+  }
+
+  frame .myload.l.listbox2
+  listbox .myload.l.listbox2.list -listvariable myload_files2 \
+    -yscrollcommand ".myload.l.listbox2.yscroll set" -selectmode browse
+  scrollbar .myload.l.listbox2.yscroll -command ".myload.l.listbox2.list yview"
+  pack  .myload.l.listbox2.yscroll -side right -fill y
+  pack  .myload.l.listbox2.list -side right  -fill both -expand true
+
+  # pack  .myload.l.listbox1 .myload.l.listbox2 -side left -fill both -expand true
+  .myload.l  add .myload.l.listbox1 -minsize 100
+  .myload.l  add .myload.l.listbox2 -minsize 100
+ 
+  frame .myload.buttons
+  button .myload.buttons.ok -text OK -command { set myload_retval [.myload.buttons.entry get]; destroy .myload} 
+  button .myload.buttons.cancel -text Cancel -command {set myload_retval {}; destroy .myload}
+  button .myload.buttons.home -text Home -command {
+    set myload_files1 $pathlist
+    set myload_index1 0
+    set myload_dir1 [abs_sym_path [.myload.l.listbox1.list get $myload_index1]]
+    set myload_files2 [lsort [glob -directory $myload_dir1 -tails \{.*,*\}]]
+    myload_set_colors
+    .myload.buttons.entry delete 0 end
+    .myload.l.listbox1.list selection set $myload_index1
+  }
+  label .myload.buttons.label  -text {File:}
+  entry .myload.buttons.entry
+  button .myload.buttons.up -text UP -command {
+    set dir2 {..}
+    set d [string replace [file normalize "$myload_dir1/$dir2/__xxx__"] end-7 end {}]
+    if { [file isdirectory $d]} {
+      myload_set_home $d
+      set myload_files2 [lsort [glob -directory $d -tails \{.*,*\}]]
+      myload_set_colors
+      set myload_dir1 $d
+      .myload.buttons.entry delete 0 end
+    }
+  }
+  pack .myload.buttons.ok .myload.buttons.up .myload.buttons.cancel \
+       .myload.buttons.home .myload.buttons.label -side left
+  pack .myload.buttons.entry -side left -fill x -expand true
+  pack .myload.l -side top -expand true -fill both
+  pack .myload.buttons -side top -fill x
+  myload_set_home $initdir
+  bind .myload <Return> { 
+    set myload_retval [.myload.buttons.entry get]
+    if {$myload_retval ne {} } {
+      destroy .myload
+    }
+  }
+  bind .myload.l.listbox2.list <Double-Button-1> {
+    set myload_retval [.myload.buttons.entry get]
+    if {$myload_retval ne {} } {
+      destroy .myload
+    }
+  }
+  bind .myload <Escape> { set myload_retval {}; destroy .myload}
+
+
+  bind .myload <Configure> {
+    set myload_default_geometry [wm geometry .myload]
+    regsub {\+.*} $myload_default_geometry {} myload_default_geometry
+  }
+
+
+  wm geometry .myload "${myload_default_geometry}"
+  update
+  set myload_dir1 [abs_sym_path [.myload.l.listbox1.list get $myload_index1]]
+  set myload_files2 [lsort [glob -directory $myload_dir1 -tails \{.*,*\}]]
+  myload_set_colors
+  bind .myload.l.listbox2.list <<ListboxSelect>> {
+    set sel [.myload.l.listbox2.list curselection]
+    if { $sel ne {} } {
+      set myload_dir1 [abs_sym_path [.myload.l.listbox1.list get $myload_index1]]
+      set dir2 [.myload.l.listbox2.list get $sel]
+      set d [string replace [file normalize "$myload_dir1/$dir2/__xxx__"] end-7 end {}]
+
+
+
+
+      if { [file isdirectory $d]} {
+        myload_set_home $d
+        set myload_files2 [lsort [glob -directory $d -tails \{.*,*\}]]
+        myload_set_colors
+        set myload_dir1 $d
+        .myload.buttons.entry delete 0 end
+      } else {
+        .myload.buttons.entry delete 0 end
+        .myload.buttons.entry insert 0 $dir2
+      }
+    }
+  }
+
+  tkwait window .myload
+
+  if { $myload_retval ne {} } {
+    set initdir "$myload_dir1"
+    return [file normalize "$myload_dir1/$myload_retval"]
+  } else {
+    return {}
+  }
+}
+
+
 # used in scheduler.c  20121111
 # get last 2 path components: example /aaa/bbb/ccc/ddd.sch -> ccc/ddd
 # optionally with extension if present and $ext==1
-proc get_cell {s {ext 0} } {
+proc get_cell {s {ext 1} } {
   set slist [file split $s]
   if { [llength $slist] >1 } {
     if {$ext} {
@@ -523,12 +726,12 @@ proc create_pins {} {
   # viewdata $retval
   set pcnt 0
   set y 0
-  set fd [open $env(HOME)/.clipboard.sch "w"]
+  set fd [open $env(HOME)/.xschem_clipboard.sch "w"]
   foreach i $lines { 
-    puts $fd "C \{devices/[lindex $i 1]\} 0 [set y [expr $y-20]]  0 0 \{ name=p[incr pcnt] lab=[lindex $i 0] \}"
+    puts $fd "C \{[rel_sym_path devices/[lindex $i 1].sym]\} 0 [set y [expr $y-20]]  0 0 \{ name=p[incr pcnt] lab=[lindex $i 0] \}"
   }
   close $fd
-  xschem merge $env(HOME)/.clipboard.sch
+  xschem merge $env(HOME)/.xschem_clipboard.sch
 }
 
 proc rectorder {x1 y1 x2 y2} {
@@ -556,9 +759,9 @@ proc schpins_to_sympins {} {
   global env
   set pinhsize 2.5
   xschem copy
-  set clipboard [read_data_nonewline $env(HOME)/.clipboard.sch]
+  set clipboard [read_data_nonewline $env(HOME)/.xschem_clipboard.sch]
   set lines [split $clipboard \n]
-  set fd [open $env(HOME)/.clipboard.sch "w"]
+  set fd [open $env(HOME)/.xschem_clipboard.sch "w"]
   foreach i $lines {
     if {[regexp {devices/(i|o|io)pin} [lindex $i 1]]} {
       if {[regexp {devices/ipin} [lindex $i 1]]} { set dir in }
@@ -617,12 +820,12 @@ proc add_lab_no_prefix {} {
   # viewdata $retval
   set pcnt 0
   set y 0
-  set fd [open $env(HOME)/.clipboard.sch "w"]
+  set fd [open $env(HOME)/.xschem_clipboard.sch "w"]
   foreach i $lines {
-    puts $fd "C \{devices/lab_pin\} 0 [set y [expr $y+20]]  0 0 \{ name=p[incr pcnt] verilog_type=wire lab=[lindex $i 0] \}"
+    puts $fd "C \{[rel_sym_path devices/lab_pin.sym]\} 0 [set y [expr $y+20]]  0 0 \{ name=p[incr pcnt] verilog_type=wire lab=[lindex $i 0] \}"
   }
   close $fd
-  xschem merge $env(HOME)/.clipboard.sch
+  xschem merge $env(HOME)/.xschem_clipboard.sch
 }
 
 
@@ -637,18 +840,18 @@ proc add_lab_prefix {} {
   # viewdata $retval
   set pcnt 0
   set y 0
-  set fd [open $env(HOME)/.clipboard.sch "w"]
+  set fd [open $env(HOME)/.xschem_clipboard.sch "w"]
   foreach i $lines {
-    puts $fd "C \{devices/lab_pin\} 0 [set y [expr $y+20]]  0 0 \{ name=p[incr pcnt] verilog_type=reg lab=i[lindex $i 0] \}"
+    puts $fd "C \{[rel_sym_path devices/lab_pin.sym]\} 0 [set y [expr $y+20]]  0 0 \{ name=p[incr pcnt] verilog_type=reg lab=i[lindex $i 0] \}"
   }
   close $fd
-  xschem merge $env(HOME)/.clipboard.sch
+  xschem merge $env(HOME)/.xschem_clipboard.sch
 }
 
 
 proc make_symbol {name} {
  global XSCHEM_SHAREDIR symbol_width
- set name [abs_sym_path $name .sch]
+ set name [abs_sym_path $name ]
  # puts "make_symbol{}, executing: ${XSCHEM_SHAREDIR}/make_sym.awk $symbol_width ${name}"
  eval exec "awk -f ${XSCHEM_SHAREDIR}/make_sym.awk $symbol_width {$name}"
  return {}
@@ -843,7 +1046,7 @@ proc about {} {
 
 proc property_search {} {
   global search_value
-  global search_substring
+  global search_exact
   global search_select
   global custom_token
 
@@ -879,13 +1082,13 @@ proc property_search {} {
         set custom_token [.lw.custom.e get]
         if $tcl_debug<=-1 then { puts stderr "|$custom_token|" }
         set token $custom_token
-        if { $search_substring==1 } { xschem search sub $search_select $token $search_value
-        } else { xschem search nosub $search_select $token $search_value }
+        if { $search_exact==1 } { xschem search exact $search_select $token $search_value
+        } else { xschem search regex $search_select $token $search_value }
         destroy .lw 
   }
   button .lw.but.cancel -text Cancel -command { destroy .lw }
-  checkbutton .lw.but.sub -text Substring -variable search_substring
-  radiobutton .lw.but.nosel -text {No selection} -variable search_select -value 0
+  checkbutton .lw.but.sub -text Exact_search -variable search_exact
+  radiobutton .lw.but.nosel -text {Highlight} -variable search_select -value 0
   radiobutton .lw.but.sel -text {Select} -variable search_select -value 1
   # 20171211 added unselect
   radiobutton .lw.but.unsel -text {Unselect} -variable search_select -value -1
@@ -1129,7 +1332,7 @@ proc change_color {} {
 proc edit_prop {txtlabel} {
    global edit_prop_default_geometry
    global prev_symbol retval symbol rcode rbutton1 rbutton2 copy_cell tcl_debug editprop_semaphore
-   global user_wants_copy_cell
+   global user_wants_copy_cell editprop_sympath
    set user_wants_copy_cell 0
    set rcode {}
    set editprop_semaphore 1
@@ -1147,7 +1350,10 @@ proc edit_prop {txtlabel} {
    ## not honored by fvwm ... 20110322
    # wm attributes .ent2 -topmost 1
    ## ... use alternate method instead 20110322
-   bind .ent2 <Visibility> { if { [regexp Obscured %s] } {raise .ent2; if { $tcl_version > 8.4 } {wm attributes  .ent2 -topmost 1} } } 
+
+   
+   #bind .ent2 <Visibility> { if { [regexp Obscured %s] } {raise .ent2; if { $tcl_version > 8.4 } {wm attributes  .ent2 -topmost 1} } } 
+   bind .ent2 <Visibility> { if { [regexp Obscured %s] } {raise .ent2}  }
    ## 
 
    # 20160325 change and remember widget size
@@ -1159,7 +1365,11 @@ proc edit_prop {txtlabel} {
    }
 
    set prev_symbol $symbol
-   label .ent2.l1  -text $txtlabel
+   set editprop_sympath [file dirname [abs_sym_path $symbol]]
+   frame .ent2.f4
+   label .ent2.f4.l1  -text $txtlabel
+   label .ent2.f4.path  -text "Path:"
+   entry .ent2.f4.e1  -textvariable editprop_sympath -width 0 -state readonly
    text .ent2.e1   -yscrollcommand ".ent2.yscroll set" -setgrid 1 \
                    -xscrollcommand ".ent2.xscroll set" -wrap none
      .ent2.e1 delete 1.0 end
@@ -1172,29 +1382,42 @@ proc edit_prop {txtlabel} {
    label .ent2.f1.l2 -text "Symbol"
    entry .ent2.f1.e2 -width 30
    .ent2.f1.e2 insert 0 $symbol
+   button .ent2.f1.b5 -text "BROWSE" -command {
+     bind .ent2 <Visibility> {}
+     set r [save_file_dialog  {New symbol} .sym INITIALINSTDIR {} 0]
+     if {$r ne {} } {
+       .ent2.f1.e2 delete 0 end
+       .ent2.f1.e2 insert 0 $r
+     }
+     raise .ent2
+     bind .ent2 <Visibility> { if { [regexp Obscured %s] } {raise .ent2}  }
+   }
+
    button .ent2.f1.b1 -text "OK" -command   {
      set retval [.ent2.e1 get 1.0 {end - 1 chars}] 
-     set symbol [ .ent2.f1.e2 get]
+     set abssymbol [abs_sym_path [ .ent2.f1.e2 get]]
+     set symbol [.ent2.f1.e2 get]
      set rcode {ok}
      set editprop_semaphore 0
      set user_wants_copy_cell $copy_cell
-     if { ($symbol ne $prev_symbol) && $copy_cell } {
-       set symbol [abs_sym_path $symbol .sym]
-       set prev_symbol [abs_sym_path $prev_symbol .sym]
+     set prev_symbol [abs_sym_path $prev_symbol]
+     if { ($abssymbol ne $prev_symbol) && $copy_cell } {
+       # puts "$abssymbol   $prev_symbol"
        if { [file exists "[file rootname $prev_symbol].sch"] } {
-         if { ! [file exists "[file rootname ${symbol}].sch"] } {
-           file copy "[file rootname $prev_symbol].sch" "[file rootname $symbol].sch"
-           # puts "file copy [file rootname $prev_symbol].sch [file rootname $symbol].sch"
+         if { ! [file exists "[file rootname ${abssymbol}].sch"] } {
+           file copy "[file rootname $prev_symbol].sch" "[file rootname $abssymbol].sch"
+           # puts "file copy [file rootname $prev_symbol].sch [file rootname $abssymbol].sch"
          }
        }
        if { [file exists "$prev_symbol"] } {
-         if { ! [file exists "$symbol"] } {
-           file copy "[file rootname $prev_symbol].sym" "[file rootname $symbol].sym"
-           # puts "file copy [file rootname $prev_symbol].sym [file rootname $symbol].sym"
+         if { ! [file exists "$abssymbol"] } {
+           file copy "$prev_symbol" "$abssymbol"
+           # puts "file copy [file rootname $prev_symbol].sym [file rootname $abssymbol].sym"
          }
        }
+       ## 20190326
      }
-     # puts "symbol: $symbol , prev_symbol: $prev_symbol"
+     #puts "symbol: $symbol , prev_symbol: $prev_symbol"
      set copy_cell 0 ;# 20120919
    }
    button .ent2.f1.b2 -text "Cancel" -command  {
@@ -1216,8 +1439,11 @@ proc edit_prop {txtlabel} {
    checkbutton .ent2.f2.r2 -text "preserve unchanged props" -variable rbutton2 -state normal
    checkbutton .ent2.f2.r3 -text "copy cell" -variable copy_cell -state normal
 
-   pack .ent2.f1.l2 .ent2.f1.e2 .ent2.f1.b1 .ent2.f1.b2 .ent2.f1.b3 .ent2.f1.b4 -side left -expand 1
-   pack .ent2.l1 -side top -fill x 
+   pack .ent2.f1.l2 .ent2.f1.e2 .ent2.f1.b1 .ent2.f1.b2 .ent2.f1.b3 .ent2.f1.b4 .ent2.f1.b5 -side left -expand 1
+   pack .ent2.f4 -side top  -anchor nw
+   #pack .ent2.f4.path .ent2.f4.e1 .ent2.f4.l1 -side left -fill x 
+   pack .ent2.f4.path -side left
+   pack .ent2.f4.e1 -side left 
    pack .ent2.f1 .ent2.f2 -side top -fill x 
    pack .ent2.f2.r1 -side left
    pack .ent2.f2.r2 -side left
@@ -1244,6 +1470,7 @@ proc edit_prop {txtlabel} {
        xschem update_symbol ok 
        set editprop_semaphore 1
        xschem fill_symbol_editprop_form
+       set editprop_sympath [file dirname [abs_sym_path $symbol]]
     
        # 20160423 no more setected stuff--> close
        if {$editprop_semaphore==0 } {
@@ -1550,16 +1777,10 @@ proc viewdata {data} {
 proc rel_sym_path {symbol} {
   global pathlist current_dirname
 
-  set extension 1
-  set symbol_orig [file rootname $symbol]
-  set ext [file extension $symbol]
-  if {![string compare $ext {}] } { 
-    set ext .sym  
-    set extension 0
-  }
-  set symbol [file rootname [file normalize $symbol]]
+  set symbol_orig $symbol
+  set symbol [file normalize $symbol]
   set lib_cell [get_cell $symbol]
-  set cell [file rootname [file tail $symbol]]
+  set cell [file tail $symbol]
   set name {}
   foreach path_elem $pathlist {
     if { ![string compare $path_elem .]  && [info exist current_dirname]} {
@@ -1570,79 +1791,90 @@ proc rel_sym_path {symbol} {
     if { [file exists [file dirname "${path_elem}/${lib_cell}"]] && 
        (![string compare $symbol_orig $lib_cell ]) } {
       set name ${lib_cell}
-      break
     # /.../path/.../libname/cellname[.ext] and libname in $path_elem 
     # --> libname/cellname
     } elseif { (![string compare $symbol  [file normalize "${path_elem}/${lib_cell}"] ]) 
              && [file exists [file dirname "${path_elem}/${lib_cell}"]] } {
       set name ${lib_cell}
-      break
-    } 
-    # symname[.ext]
-    # --> symname.ext
-    if { [file exists "${path_elem}/${cell}${ext}"] } {
-      set name ${cell}${ext}
-      break
-    # /.../path/.../libname/symname[.ext] and libname in XSCHEM_LIBRARY_PATH 
-    # --> symname.ext
+    # symname
+    # --> symname
+    } elseif { [file exists "${path_elem}/${cell}"] } {
+      set name ${cell}
+    # /.../path/.../libname/symname and libname in XSCHEM_LIBRARY_PATH 
+    # --> symname
     } elseif { (![string compare $symbol [file normalize "${path_elem}/${cell}"] ]) 
-             && [file exists "${path_elem}/${cell}${ext}"] } {
-      set name ${cell}${ext}
-      break
+             && [file exists "${path_elem}/${cell}"] } {
+      set name ${cell}
     } 
+    if {$name ne {} } { break} 
   }
   if { ![string compare $name {} ] } {
     # no known lib, so return full path
-    set name ${symbol}$ext
-  }
-  if {$extension == 0 } {
-    return [file rootname $name] 
-  } else {
-    return $name
+    set name ${symbol_orig}
   }
+  return $name
 }
 
 
 # given a library/symbol return its absolute path
-proc abs_sym_path {fname {required_ext {}} } {
+proc abs_sym_path {fname {ext {} } } {
   global pathlist current_dirname
 
+  # add extension for 1.0 file format compatibility
+  if { $ext ne {} } { 
+    set fname [file rootname $fname]$ext
+  }
+
+  if {$fname eq {} } return {}
   set name {}
   # fname is of type libname/cellname[.ext] but not ./cellname[.ext] or
   # ../cellname[.ext] and has a slash, so no cellname[.ext] 
-  if {![string compare [file rootname $fname] [get_cell $fname] ] } {
+  if {![string compare $fname [get_cell $fname] ] } {
     foreach path_elem $pathlist {
       if { ![string compare $path_elem .]  && [info exist current_dirname]} {
         set path_elem $current_dirname
       }
-      # libname/cellname[.ext] and libname is in pathlist
-      # --> normalized $pathlist/libname/cellname$required_ext
-      # cellname[.ext] and $pathlist/cellname$required_ext exists
-      # --> normalized $pathlist/cellname.$required_ext
+      # libname/cellname and libname is in pathlist
+      # --> normalized $pathlist/libname/cellname
+      # cellname and $pathlist/cellname exists
+      # --> normalized $pathlist/cellname
       if { ([file exists "${path_elem}/[file dirname $fname]"] ) &&
         [regexp {\/} $fname] 
       } {
         #puts here1
-        set name  [file normalize "$path_elem/[get_cell $fname]$required_ext"]
+        set name  [file normalize "$path_elem/[get_cell $fname]"]
         break
       }
-      if { [file exists "${path_elem}/[file rootname ${fname}]${required_ext}"] &&
+      if { [file exists "${path_elem}/${fname}"] &&
         ![regexp {\/} $fname] 
       } {
         #puts here2
-        set name  [file normalize "$path_elem/[get_cell $fname]$required_ext"]
+        set name  [file normalize "$path_elem/[get_cell $fname]"]
         break
       }
     }
   }
 
   if { ![string compare $name {}] } {
-    #puts here3
-    set name [file rootname [file normalize $fname]]$required_ext
+#    if { ![string compare $fname [file tail $fname]] } {
+#      set name [file normalize ${current_dirname}/$fname]
+#    } else {
+#      set name [file normalize $fname]
+#    }
+     set name $fname
   }
   return $name
 }
 
+proc add_ext {fname ext} {
+#  if {![string match "*$ext" $fname]} {
+#    set fname ${fname}$ext
+#  }
+#  return $fname
+  return [file rootname $fname]$ext
+}
+ 
+
 proc input_number {txt cmd} {
           global xx
           toplevel .lw -class Dialog
@@ -1729,8 +1961,6 @@ proc get_file_path {ff} {
 ###
 
 
-
-
 # tcl variable XSCHEM_LIBRARY_PATH  should already be set in xschemrc
 set pathlist {}
 if { [info exists XSCHEM_LIBRARY_PATH] } {
@@ -1773,6 +2003,10 @@ set tclcmd_txt {}
 ###
 ### user preferences: set default values
 ###
+
+## list of tcl procedures to load at end of xschem.tcl
+set_ne tcl_files {}
+set_ne use_list_dirs {1}
 set_ne netlist_dir {.}
 set_ne hspice_netlist 0
 set_ne verilog_2001 1
@@ -1821,6 +2055,7 @@ set_ne to_png {gm convert}
 ## 20160325 remember edit_prop widget size
 set_ne edit_prop_default_geometry 80x12
 set_ne text_line_default_geometry 80x12
+set_ne myload_default_geometry 600x300
 set_ne terminal xterm
 
 # set_ne analog_viewer waveview
@@ -1983,7 +2218,7 @@ foreach i $pathlist  {
 set txt ""
 set custom_token {lab}
 set search_value {}
-set search_substring 0
+set search_exact 0
 
 
 # 20171005
@@ -2168,8 +2403,8 @@ font configure Underline-Font -underline true -size 24
    .menubar.option.menu add radiobutton -label "tEDAx netlist" -variable netlist_type -value tedax \
         -accelerator {Shift+V} \
         -command "xschem netlist_type tedax"
-   .menubar.edit.menu add command -label "Undo" -state disabled -accelerator U
-   .menubar.edit.menu add command -label "Redo" -state disabled -accelerator {Ctrl+R}
+   .menubar.edit.menu add command -label "Undo" -command "xschem undo; xschem redraw" -accelerator U
+   .menubar.edit.menu add command -label "Redo" -command "xschem redo; xschem redraw" -accelerator {Shift+U}
    .menubar.edit.menu add command -label "Copy" -command "xschem copy" -accelerator Ctrl+C
    .menubar.edit.menu add command -label "Cut" -command "xschem cut"   -accelerator Ctrl+X
    .menubar.edit.menu add command -label "Paste" -command "xschem paste" -accelerator Ctrl+V
@@ -2464,3 +2699,7 @@ font configure Underline-Font -underline true -size 24
    }
 }
 
+
+foreach i $tcl_files {
+  source $i
+}
